@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2011  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -41,44 +41,52 @@ static GSList *g_drivers = NULL;
 
 /* 27.007 Section 7.7 */
 enum clir_status {
-	CLIR_STATUS_NOT_PROVISIONED = 0,
-	CLIR_STATUS_PROVISIONED_PERMANENT,
-	CLIR_STATUS_UNKNOWN,
-	CLIR_STATUS_TEMPORARY_RESTRICTED,
-	CLIR_STATUS_TEMPORARY_ALLOWED
+	CLIR_STATUS_NOT_PROVISIONED =		0,
+	CLIR_STATUS_PROVISIONED_PERMANENT =	1,
+	CLIR_STATUS_UNKNOWN =			2,
+	CLIR_STATUS_TEMPORARY_RESTRICTED =	3,
+	CLIR_STATUS_TEMPORARY_ALLOWED =		4
 };
 
 /* 27.007 Section 7.6 */
 enum clip_status {
-	CLIP_STATUS_NOT_PROVISIONED = 0,
-	CLIP_STATUS_PROVISIONED,
-	CLIP_STATUS_UNKNOWN
+	CLIP_STATUS_NOT_PROVISIONED =		0,
+	CLIP_STATUS_PROVISIONED =		1,
+	CLIP_STATUS_UNKNOWN =			2
 };
 
 /* 27.007 Section 7.30 */
 enum cnap_status {
-	CNAP_STATUS_NOT_PROVISIONED = 0,
-	CNAP_STATUS_PROVISIONED,
-	CNAP_STATUS_UNKNOWN
+	CNAP_STATUS_NOT_PROVISIONED =		0,
+	CNAP_STATUS_PROVISIONED =		1,
+	CNAP_STATUS_UNKNOWN =			2
 };
 
 /* 27.007 Section 7.8 */
 enum colp_status {
-	COLP_STATUS_NOT_PROVISIONED = 0,
-	COLP_STATUS_PROVISIONED = 1,
-	COLP_STATUS_UNKNOWN = 2
+	COLP_STATUS_NOT_PROVISIONED =		0,
+	COLP_STATUS_PROVISIONED =		1,
+	COLP_STATUS_UNKNOWN =			2
+};
+
+/* 27.007 Section 7.9 */
+enum cdip_status {
+	CDIP_STATUS_NOT_PROVISIONED =		0,
+	CDIP_STATUS_PROVISIONED =		1,
+	CDIP_STATUS_UNKNOWN =			2
 };
 
 /* This is not defined in 27.007, but presumably the same as CLIP/COLP */
 enum colr_status {
-	COLR_STATUS_NOT_PROVISIONED = 0,
-	COLR_STATUS_PROVISIONED = 1,
-	COLR_STATUS_UNKNOWN = 2
+	COLR_STATUS_NOT_PROVISIONED =		0,
+	COLR_STATUS_PROVISIONED =		1,
+	COLR_STATUS_UNKNOWN =			2
 };
 
 enum call_setting_type {
 	CALL_SETTING_TYPE_CLIP = 0,
 	CALL_SETTING_TYPE_CNAP,
+	CALL_SETTING_TYPE_CDIP,
 	CALL_SETTING_TYPE_COLP,
 	CALL_SETTING_TYPE_COLR,
 	CALL_SETTING_TYPE_CLIR,
@@ -90,6 +98,7 @@ struct ofono_call_settings {
 	int colr;
 	int clip;
 	int cnap;
+	int cdip;
 	int colp;
 	int clir_setting;
 	int cw;
@@ -117,6 +126,18 @@ static const char *clip_status_to_string(int status)
 	return "unknown";
 }
 
+static const char *cdip_status_to_string(int status)
+{
+	switch (status) {
+	case CDIP_STATUS_NOT_PROVISIONED:
+		return "disabled";
+	case CDIP_STATUS_PROVISIONED:
+		return "enabled";
+	}
+
+	return "unknown";
+}
+
 static const char *cnap_status_to_string(int status)
 {
 	switch (status) {
@@ -227,6 +248,28 @@ static void set_clir_override(struct ofono_call_settings *cs, int override)
 						DBUS_TYPE_STRING, &str);
 }
 
+static void set_cdip(struct ofono_call_settings *cs, int cdip)
+{
+	DBusConnection *conn;
+	const char *path;
+	const char *str;
+
+	if (cs->cdip == cdip)
+		return;
+
+	cs->cdip = cdip;
+
+	conn = ofono_dbus_get_connection();
+	path = __ofono_atom_get_path(cs->atom);
+
+	str = cdip_status_to_string(cdip);
+
+	ofono_dbus_signal_property_changed(conn, path,
+						OFONO_CALL_SETTINGS_INTERFACE,
+						"CalledLinePresentation",
+						DBUS_TYPE_STRING, &str);
+}
+
 static void set_clip(struct ofono_call_settings *cs, int clip)
 {
 	DBusConnection *conn;
@@ -289,7 +332,7 @@ static void set_colp(struct ofono_call_settings *cs, int colp)
 
 	ofono_dbus_signal_property_changed(conn, path,
 						OFONO_CALL_SETTINGS_INTERFACE,
-						"CalledLinePresentation",
+						"ConnectedLinePresentation",
 						DBUS_TYPE_STRING, &str);
 }
 
@@ -311,7 +354,7 @@ static void set_colr(struct ofono_call_settings *cs, int colr)
 
 	ofono_dbus_signal_property_changed(conn, path,
 						OFONO_CALL_SETTINGS_INTERFACE,
-						"CalledLineRestriction",
+						"ConnectedLineRestriction",
 						DBUS_TYPE_STRING, &str);
 }
 
@@ -434,9 +477,10 @@ static void cw_ss_set_callback(const struct ofono_error *error, void *data)
 	struct ofono_call_settings *cs = data;
 
 	if (error->type != OFONO_ERROR_TYPE_NO_ERROR) {
-		DBG("setting CW via SS failed");
+		DBG("setting CW via SS failed with error: %s",
+			telephony_error_to_str(error));
 		__ofono_dbus_pending_reply(&cs->pending,
-					__ofono_error_failed(cs->pending));
+			__ofono_error_from_error(error, cs->pending));
 
 		return;
 	}
@@ -571,9 +615,10 @@ static void clip_cnap_colp_colr_ss_query_cb(const struct ofono_error *error,
 	const char *value;
 
 	if (error->type != OFONO_ERROR_TYPE_NO_ERROR) {
-		DBG("Error occurred during ss control query");
+		DBG("SS control query failed with error: %s",
+			telephony_error_to_str(error));
 		__ofono_dbus_pending_reply(&cs->pending,
-					__ofono_error_failed(cs->pending));
+			__ofono_error_from_error(error, cs->pending));
 
 		return;
 	}
@@ -595,13 +640,13 @@ static void clip_cnap_colp_colr_ss_query_cb(const struct ofono_error *error,
 	case CALL_SETTING_TYPE_COLP:
 		set_colp(cs, status);
 		value = colp_status_to_string(status);
-		context = "CalledLinePresentation";
+		context = "ConnectedLinePresentation";
 		break;
 
 	case CALL_SETTING_TYPE_COLR:
 		set_colr(cs, status);
 		value = colr_status_to_string(status);
-		context = "CalledLineRestriction";
+		context = "ConnectedLineRestriction";
 		break;
 
 	default:
@@ -690,7 +735,7 @@ static void clir_ss_query_callback(const struct ofono_error *error,
 
 	switch (network) {
 	case CLIR_STATUS_UNKNOWN:
-		value = "uknown";
+		value = "unknown";
 		break;
 
 	case CLIR_STATUS_PROVISIONED_PERMANENT:
@@ -729,9 +774,10 @@ static void clir_ss_set_callback(const struct ofono_error *error, void *data)
 	struct ofono_call_settings *cs = data;
 
 	if (error->type != OFONO_ERROR_TYPE_NO_ERROR) {
-		DBG("setting clir via SS failed");
+		DBG("setting clir via SS failed with error: %s",
+			telephony_error_to_str(error));
 		__ofono_dbus_pending_reply(&cs->pending,
-					__ofono_error_failed(cs->pending));
+			__ofono_error_from_error(error, cs->pending));
 
 		return;
 	}
@@ -825,7 +871,7 @@ static void cs_register_ss_controls(struct ofono_call_settings *cs)
 
 	__ofono_ussd_ssc_register(cs->ussd, "43", cw_ss_control, cs, NULL);
 
-	if (cs->driver->colr_query)
+	if (cs->driver->colr_query != NULL)
 		__ofono_ussd_ssc_register(cs->ussd, "77",
 					clip_cnap_colp_colr_ss, cs, NULL);
 }
@@ -839,7 +885,7 @@ static void cs_unregister_ss_controls(struct ofono_call_settings *cs)
 
 	__ofono_ussd_ssc_unregister(cs->ussd, "43");
 
-	if (cs->driver->colr_query)
+	if (cs->driver->colr_query != NULL)
 		__ofono_ussd_ssc_unregister(cs->ussd, "77");
 }
 
@@ -870,16 +916,20 @@ static DBusMessage *generate_get_properties_reply(struct ofono_call_settings *cs
 	ofono_dbus_dict_append(&dict, "CallingLinePresentation",
 				DBUS_TYPE_STRING, &str);
 
-	str = cnap_status_to_string(cs->clip);
+	str = cnap_status_to_string(cs->cnap);
 	ofono_dbus_dict_append(&dict, "CallingNamePresentation",
 				DBUS_TYPE_STRING, &str);
 
 	str = colp_status_to_string(cs->colp);
-	ofono_dbus_dict_append(&dict, "CalledLinePresentation",
+	ofono_dbus_dict_append(&dict, "ConnectedLinePresentation",
 				DBUS_TYPE_STRING, &str);
 
 	str = colr_status_to_string(cs->colr);
-	ofono_dbus_dict_append(&dict, "CalledLineRestriction",
+	ofono_dbus_dict_append(&dict, "ConnectedLineRestriction",
+				DBUS_TYPE_STRING, &str);
+
+	str = cdip_status_to_string(cs->cdip);
+	ofono_dbus_dict_append(&dict, "CalledLinePresentation",
 				DBUS_TYPE_STRING, &str);
 
 	str = clir_status_to_string(cs->clir);
@@ -934,6 +984,28 @@ static void query_clir(struct ofono_call_settings *cs)
 	cs->driver->clir_query(cs, cs_clir_callback, cs);
 }
 
+static void cs_cdip_callback(const struct ofono_error *error,
+				int state, void *data)
+{
+	struct ofono_call_settings *cs = data;
+
+	if (error->type == OFONO_ERROR_TYPE_NO_ERROR)
+		set_cdip(cs, state);
+
+	query_clir(cs);
+}
+
+static void query_cdip(struct ofono_call_settings *cs)
+{
+	if (cs->driver->cdip_query == NULL) {
+		query_clir(cs);
+		return;
+	}
+
+	cs->driver->cdip_query(cs, cs_cdip_callback, cs);
+}
+
+
 static void cs_cnap_callback(const struct ofono_error *error,
 				int state, void *data)
 {
@@ -942,13 +1014,13 @@ static void cs_cnap_callback(const struct ofono_error *error,
 	if (error->type == OFONO_ERROR_TYPE_NO_ERROR)
 		set_cnap(cs, state);
 
-	query_clir(cs);
+	query_cdip(cs);
 }
 
 static void query_cnap(struct ofono_call_settings *cs)
 {
 	if (cs->driver->cnap_query == NULL) {
-		query_clir(cs);
+		query_cdip(cs);
 		return;
 	}
 
@@ -1110,11 +1182,11 @@ static DBusMessage *set_clir(DBusMessage *msg, struct ofono_call_settings *cs,
 		return __ofono_error_not_implemented(msg);
 
 	if (!strcmp(setting, "default"))
-		clir = 0;
+		clir = CLIR_STATUS_NOT_PROVISIONED;
 	else if (!strcmp(setting, "enabled"))
-		clir = 1;
+		clir = CLIR_STATUS_PROVISIONED_PERMANENT;
 	else if (!strcmp(setting, "disabled"))
-		clir = 2;
+		clir = CLIR_STATUS_UNKNOWN;
 
 	if (clir == -1)
 		return __ofono_error_invalid_format(msg);
@@ -1261,16 +1333,19 @@ static DBusMessage *cs_set_property(DBusConnection *conn, DBusMessage *msg,
 	return __ofono_error_invalid_args(msg);
 }
 
-static GDBusMethodTable cs_methods[] = {
-	{ "GetProperties",	"",	"a{sv}",	cs_get_properties,
-							G_DBUS_METHOD_FLAG_ASYNC },
-	{ "SetProperty",	"sv",	"",		cs_set_property,
-							G_DBUS_METHOD_FLAG_ASYNC },
+static const GDBusMethodTable cs_methods[] = {
+	{ GDBUS_ASYNC_METHOD("GetProperties",
+				NULL, GDBUS_ARGS({ "properties", "a{sv}" }),
+				cs_get_properties) },
+	{ GDBUS_ASYNC_METHOD("SetProperty",
+			GDBUS_ARGS({ "property", "s" }, { "value", "v" }),
+			NULL, cs_set_property) },
 	{ }
 };
 
-static GDBusSignalTable cs_signals[] = {
-	{ "PropertyChanged",	"sv" },
+static const GDBusSignalTable cs_signals[] = {
+	{ GDBUS_SIGNAL("PropertyChanged",
+			GDBUS_ARGS({ "property", "s" }, { "value", "v" })) },
 	{ }
 };
 
@@ -1319,7 +1394,7 @@ static void call_settings_remove(struct ofono_atom *atom)
 	if (cs == NULL)
 		return;
 
-	if (cs->driver && cs->driver->remove)
+	if (cs->driver != NULL && cs->driver->remove != NULL)
 		cs->driver->remove(cs);
 
 	g_free(cs);
@@ -1342,11 +1417,11 @@ struct ofono_call_settings *ofono_call_settings_create(struct ofono_modem *modem
 		return NULL;
 
 	/* Set all the settings to unknown state */
-	cs->clip = 2;
-	cs->cnap = 2;
-	cs->clir = 2;
-	cs->colp = 2;
-	cs->colr = 2;
+	cs->clip = CLIP_STATUS_UNKNOWN;
+	cs->cnap = CNAP_STATUS_UNKNOWN;
+	cs->clir = CLIR_STATUS_UNKNOWN;
+	cs->colp = COLP_STATUS_UNKNOWN;
+	cs->colr = COLR_STATUS_UNKNOWN;
 	cs->atom = __ofono_modem_add_atom(modem, OFONO_ATOM_TYPE_CALL_SETTINGS,
 						call_settings_remove, cs);
 
@@ -1385,7 +1460,6 @@ void ofono_call_settings_register(struct ofono_call_settings *cs)
 	DBusConnection *conn = ofono_dbus_get_connection();
 	const char *path = __ofono_atom_get_path(cs->atom);
 	struct ofono_modem *modem = __ofono_atom_get_modem(cs->atom);
-	struct ofono_atom *ussd_atom;
 
 	if (!g_dbus_register_interface(conn, path,
 					OFONO_CALL_SETTINGS_INTERFACE,
@@ -1403,12 +1477,6 @@ void ofono_call_settings_register(struct ofono_call_settings *cs)
 					OFONO_ATOM_TYPE_USSD,
 					ussd_watch, cs, NULL);
 
-	ussd_atom = __ofono_modem_find_atom(modem, OFONO_ATOM_TYPE_USSD);
-
-	if (ussd_atom && __ofono_atom_get_registered(ussd_atom))
-		ussd_watch(ussd_atom, OFONO_ATOM_WATCH_CONDITION_REGISTERED,
-				cs);
-
 	__ofono_atom_register(cs->atom, call_settings_unregister);
 }
 
